@@ -1,5 +1,3 @@
-=encoding utf8
-
 =head1 NAME
 
 Locale::SubCountry - Convert state, province, county etc. names to/from ISO 3166-2 codes
@@ -10,24 +8,24 @@ Locale::SubCountry - Convert state, province, county etc. names to/from ISO 3166
     my $UK = new Locale::SubCountry($country_code);
     if ( not $UK )
     {
-        die "Invalid code $country_code\n"; 
+        die "Invalid code $country_code\n";
     }
     elsif (  $UK->has_sub_countries )
     {
         print($UK->full_name('DGY'),"\n");           # Dumfries and Galloway
         print($UK->regional_division('DGY'),"\n");   # SCT (Scotland)
     }
-    
+
     my $australia = new Locale::SubCountry('Australia');
     if ( not $australia )
     {
-        die "Invalid code: Australia\n"; 
+        die "Invalid code: Australia\n";
     }
     else
     {
         print($australia->country,"\n");        # Australia
         print($australia->country_code,"\n");   # AU
-    
+
         if (  $australia->has_sub_countries )
         {
             print($australia->code('New South Wales '),"\n");     # NSW
@@ -37,25 +35,25 @@ Locale::SubCountry - Convert state, province, county etc. names to/from ISO 3166
             print($australia->category('NSW'),"\n");              # state
             print($australia->FIPS10_4_code('ACT'),"\n");         # 01
             print($australia->ISO3166_2_code('02'),"\n");         # NSW
-      
+
             my @aus_state_names  = $australia->all_full_names;
             my @aus_code_names   = $australia->all_codes;
             my %aus_states_keyed_by_code  = $australia->code_full_name_hash;
             my %aus_states_keyed_by_name  = $australia->full_name_code_hash;
-      
+
             foreach my $code ( sort keys %aus_states_keyed_by_code )
             {
                printf("%-3s : %s\n",$code,$aus_states_keyed_by_code{$code});
-            }   
+            }
         }
     }
-    
+
     # Methods for country codes and names
-    
+
     my $world = new Locale::SubCountry::World;
     my @all_countries     = $world->all_full_names;
     my @all_country_codes = $world->all_codes;
-    
+
     my %all_countries_keyed_by_name = $world->full_name_code_hash;
     my %all_country_keyed_by_code   = $world->code_full_name_hash;
 
@@ -68,7 +66,8 @@ can also be done. Sub country codes are defined in "ISO 3166-2:2007,
 Codes for the representation of names of countries and their subdivisions".
 
 Sub countries are termed as states in the US and Australia, provinces
-in Canada and counties in the UK and Ireland.
+in Canada and counties in the UK and Ireland. Ohter terms include region,
+department, city and territory.
 
 Names and ISO 3166-2 codes for all sub countries in a country can be
 returned as either a hash or an array.
@@ -76,7 +75,7 @@ returned as either a hash or an array.
 Names and ISO 3166-1 codes for all countries in the world can be
 returned as either a hash or an array.
 
-ISO 3166-2 codes can be converted to FIPS 10-4 codes. The reverse lookup can 
+ISO 3166-2 codes can be converted to FIPS 10-4 codes. The reverse lookup can
 also be done.
 
 =head1 METHODS
@@ -117,7 +116,7 @@ keyed by country code.
 
 =head2 all_full_names (for world objects)
 
-Given a world object, returns an array of all country full names, 
+Given a world object, returns an array of all country full names,
 sorted alphabetically.
 
 =head2 all_codes (for world objects)
@@ -154,15 +153,15 @@ Returns the current country of a sub country object, the format is title case
 
 =head2 country_code
 
-Given a sub country object, returns the two letter ISO 3166-1 code of the country  
+Given a sub country object, returns the two letter ISO 3166-1 code of the country
 
 
 =head2 code
 
-Given a sub country object, the C<code> method takes the full name of a sub 
+Given a sub country object, the C<code> method takes the full name of a sub
 country and returns the sub country's ISO 3166-2 code. The full name can appear
 in mixed case. All white space and non alphabetic characters are ignored, except
-the single space used to separate sub country names such as "New South Wales". 
+the single space used to separate sub country names such as "New South Wales".
 The code is returned as a capitalised string, or "unknown" if no match is found.
 
 =head2 full_name
@@ -179,31 +178,35 @@ returned as an upper cased string.
 
 Given a sub country object, the C<category> method takes the ISO 3166-2 code of
 a sub country and returns the sub country's category type. Examples are city,
-province,state and district. The category is returned as a capitalised string, 
+province,state and district. The category is returned as a capitalised string,
 or "unknown" if no match is found.
- 
+
 =head2 regional_division
 
-Given a sub country object, the C<regional_division> method takes the 
-ISO 3166-2 code of a sub country and returns the sub country's 
-regionional_division. This is, an alphanumeric code. The regional_division 
+Given a sub country object, the C<regional_division> method takes the
+ISO 3166-2 code of a sub country and returns the sub country's
+regionional_division. This is, an alphanumeric code. The regional_division
 is returned as a capitalised string,  or "unknown" if no match is found.
 
 =head2 has_sub_countries
 
 Given a sub country object, the C<has_sub_countries> method returns 1 if the
-current country has sub countries, or 0 if it does not. Some small countries 
+current country has sub countries, or 0 if it does not. Some small countries
 such as Singapore do not have sub countries.
 
 =head2 FIPS10_4_code
 
-Given a sub country object, the C<FIPS_10_4_code> method takes the ISO 3166-2 code 
+NOTE: On September 2, 2008, FIPS 10-4 was one of ten standards withdrawn by NIST as a
+Federal Information Processing Standard. It may not be supported by this module in
+the future.
+
+Given a sub country object, the C<FIPS_10_4_code> method takes the ISO 3166-2 code
 of a sub country and returns the sub country's FIPS 10-4 code, or the string 'unknown',
 if none exists. FIPS is a standard  developed by the US government.
 
 =head2 ISO3166_2_code
 
-Given a sub country object, the C<ISO3166_2_code> method takes the FIPS 10-4 code 
+Given a sub country object, the C<ISO3166_2_code> method takes the FIPS 10-4 code
 of a sub country and returns the sub country's ISO 3166-2 code, or the string 'unknown',
 if none exists.
 
@@ -233,30 +236,34 @@ sorted alphabetically. If the country has no sub countries, returns undef.
 
 =head1 SEE ALSO
 
-ISO 3166-1:1997 Codes for the representation of names of countries and their 
-subdivisions - Part 1: Country codes 
+L<Locale::Country>,L<Lingua::EN::AddressParse>,
+L<Geo::StreetAddress::US>,L<Geo::PostalAddress>,L<Geo::IP>
+L<WWW::Scraper::Wikipedia::ISO3166> for obtaining ISO 3166-2 data
 
-ISO 3166-2:1998 Codes for the representation of names of countries and their 
-subdivisions - Part 2: Country subdivision code 
+ISO 3166-1:2007 Codes for the representation of names of countries and their
+subdivisions - Part 1: Country codes
+
+ISO 3166-2:2007 Codes for the representation of names of countries and their
+subdivisions - Part 2: Country subdivision code
 Also released as AS/NZS 2632.2:1999
 
 Federal Information Processing Standards Publication 10-4
-1995 April Specifications for  COUNTRIES, DEPENDENCIES, AREAS OF SPECIAL SOVEREIGNTY, 
-AND THEIR PRINCIPAL ADMINISTRATIVE DIVISIONS 
+1995 April Specifications for  COUNTRIES, DEPENDENCIES, AREAS OF SPECIAL SOVEREIGNTY,
+AND THEIR PRINCIPAL ADMINISTRATIVE DIVISIONS
+
+L<http://www.statoids.com/statoids.html> is a good source for sub country codes plus
+other statistical data.
 
-L<http://www.statoids.com/statoids.html>
 
 
-L<Locale::Country>,L<Lingua::EN::AddressParse>,
-L<Geo::StreetAddress::US>L<Geo::PostalAddress>L<Geo::IP>
 
 =head1 LIMITATIONS
 
-ISO 3166-2:1998 defines all sub country codes as being up to 3 letters and/or
+ISO 3166-2:2007 defines all sub country codes as being up to 3 letters and/or
 numbers. These codes are commonly accepted for countries like the USA
-and Canada. In Australia  this method of abbreviation is not widely accepted. 
-For example, the ISO code for 'New South Wales' is 'NS', but 'NSW' is the  
-abbreviation that is most commonly used. I could add a flag to enforce 
+and Canada. In Australia  this method of abbreviation is not widely accepted.
+For example, the ISO code for 'New South Wales' is 'NS', but 'NSW' is the
+abbreviation that is most commonly used. I could add a flag to enforce
 ISO-3166-2 codes if needed.
 
 The ISO 3166-2 standard romanizes the names of provinces and regions in non-latin
@@ -268,8 +275,8 @@ the correct code for that sub country. These entries are usually duplicated
 because the name represents two different types of sub country, such as a
 province and a geographical unit. Examples are:
 
-    AZERBAIJAN : Lnkran; LA (the City), LAN (the Rayon)
-    AZERBAIJAN : Ski; SA,SAK
+    AZERBAIJAN : Lankaran; LA (the City), LAN (the Rayon) [see note]
+    AZERBAIJAN : Saki; SA,SAK [see note]
     AZERBAIJAN : Susa; SS,SUS
     AZERBAIJAN : Yevlax; YE,YEV
     INDONESIA  : Kalimantan Timur; KI,KT
@@ -281,7 +288,12 @@ province and a geographical unit. Examples are:
     MOLDOVA    : Gngheni; UN,UGI
     MOZAMBIQUE : Maputo; MPM,L
 
-FIPS codes are not provided for all sub countries. 
+Note: these names are spelt with a diaeresis character (two dots) above
+some of the 'a' characters. This causes utf8 errors on some versions
+of Perl, so they are omitted here. See the Locale::SubCountry::Data module
+for correct spelling
+
+FIPS codes are not provided for all sub countries.
 
 
 =head1 AUTHOR
@@ -291,10 +303,12 @@ Locale::SubCountry was written by Kim Ryan <kimryan at cpan dot org>.
 
 =head1 CREDITS
 
+Ron Savage for many corrections to the data
+
 Alastair McKinstry provided many of the sub country codes and names.
 
 Terrence Brannon produced Locale::US, which was the starting point for
-this module. 
+this module.
 
 Mark Summerfield and Guy Fraser provided the list of UK counties.
 
@@ -303,7 +317,7 @@ TJ Mather supplied the FIPS codes and many ammendments to the sub country data
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (c) 2012 Kim Ryan. All rights reserved.
+Copyright (c) 2013 Kim Ryan. All rights reserved.
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself.
@@ -322,13 +336,12 @@ use Locale::SubCountry::Data;
 
 #-------------------------------------------------------------------------------
 
-our $VERSION = '1.56';
-
 package Locale::SubCountry::World;
+our $VERSION = '1.62';
 
-# Define all the methods for the 'world' class here. Note that because the 
+# Define all the methods for the 'world' class here. Note that because the
 # name space inherits from the Locale::SubCountry name space, the
-# package wide variables $::country_lookup and $Locale::SubCountry::subcountry_lookup are
+# package wide variables $SubCountry::country_lookup and $Locale::SubCountry::subcountry_lookup are
 # accessible.
 
 
@@ -380,11 +393,12 @@ sub all_codes
 #-------------------------------------------------------------------------------
 
 package Locale::SubCountry;
+our $VERSION = '1.62';
 
 #-------------------------------------------------------------------------------
 # Initialization code must be run first to create global data structure.
 # Read in the list of abbreviations and full names defined in the
-# Locale::SubCountryData package
+# Locale::SubCountry::Data package
 
 {
 
@@ -392,19 +406,19 @@ package Locale::SubCountry;
     {
       die "Could not locate Locale::SubCountry::Data::xml_data variable";
     }
-    
+
     # Get all the data from the Locale::SubCountryData pakage and place into an array of lines
     my @lines = split(/\n/,$Locale::SubCountry::Data::xml_data);
- 
+
     while ( @lines )
     {
         my $current_line = shift(@lines);
-       
+
         # Data is in XML format, use a simple parser to extract it
         my ($country_name,$country_code);
         if ( $current_line =~ /<country>/ )
         {
-            # start of a  /<country> .. </country> block
+            # start of a  <country> .. </country> block
             my $country_finished = 0;
             until ( $country_finished )
             {
@@ -447,17 +461,17 @@ package Locale::SubCountry;
                         }
                         elsif ( $current_line =~ /<\/subcountry>/ )
                         {
-                            
+
                             $sub_country_finished = 1;
-                            
+
                             # Some sub countries have no ISO code, such as Shariff Kabunsuan in the
                             # Phillipines. Only index sub country if it has a code
                             if ( defined $sub_country_code )
                             {
-                                # Insert into doubly indexed hash, grouped by country for ISO 3166-2 
-                                # codes. One hash is keyed by abbreviation and one by full name. Although 
+                                # Insert into doubly indexed hash, grouped by country for ISO 3166-2
+                                # codes. One hash is keyed by abbreviation and one by full name. Although
                                 # data is duplicated, this provides the fastest lookup and simplest code.
-                
+
                                 $Locale::SubCountry::subcountry_lookup{$country_name}{_code_keyed}{$sub_country_code} = $sub_country_name;
                                 $Locale::SubCountry::subcountry_lookup{$country_name}{_full_name_keyed}{$sub_country_name} = $sub_country_code;
                             }
@@ -466,7 +480,7 @@ package Locale::SubCountry;
                             {
                                 $Locale::SubCountry::subcountry_lookup{$country_name}{$sub_country_code}{_category} = $category;
                             }
-            
+
                             if ( $regional_division )
                             {
                                 $Locale::SubCountry::subcountry_lookup{$country_name}{$sub_country_code}{_regional_division} = $regional_division;
@@ -479,9 +493,9 @@ package Locale::SubCountry;
                                 $Locale::SubCountry::subcountry_lookup{$country_name}{_ISO3166_2_code_keyed}{$sub_country_code} = $FIPS_code;
                             }
                         }
-                        else
+                        elsif ( $current_line =~ /\w.*/ )
                         {
-                            die "Badly formed sub country data in $country_name\nData: $current_line\n";
+                            die "Badly formed XML sub country data in $country_name\nData: $current_line\n";
                         }
                     }
                 }
@@ -493,23 +507,18 @@ package Locale::SubCountry;
                     # The user can supply either form to create a new sub_country
                     # object, and the objects properties will hold both the countries
                     # name and it's code.
-    
+
                     $Locale::SubCountry::country_lookup{_code_keyed}{$country_code} = $country_name;
                     $Locale::SubCountry::country_lookup{_full_name_keyed}{$country_name} = $country_code;
 
                 }
-                else
+                elsif ( $current_line =~ /\w.*/ )
                 {
-                    die "Badly formed country data in $country_name\nData: $current_line\n";                   
+                    die "Badly formed XML country data in $country_name\nData: $current_line\n";
                 }
             }
-        
         }
     }
-    
-    # use Data::Dumper;
-    # print Dumper(\%{ $Locale::SubCountry::subcountry_lookup{_full_name_keyed} });
-    # die;
 }
 
 #-------------------------------------------------------------------------------
@@ -522,7 +531,7 @@ sub new
 
 
     my ($country,$country_code);
-    
+
 
     # Country may be supplied either as a two letter code, or the full name
     if ( length($country_or_code) == 2 )
@@ -727,7 +736,7 @@ sub full_name
     $code = _clean($code);
     $code = uc($code);
 
-    my $full_name = 
+    my $full_name =
         $Locale::SubCountry::subcountry_lookup{$sub_country->{_country}}{_code_keyed}{$code};
     if ( $uc_name )
     {
@@ -847,4 +856,3 @@ sub _clean
 }
 
 return(1);
-
